//*****************************************************************************
//
// File Name	: 'SED1530.h'
// Title		: SPI interface driver for the LSU7S1011A SED1530 graphic LCD
// Author		: based on code for E0855-2 by Marc Meise, adapted for LSU7S1011A by Peter Lampe
// Adapted		; Septempber 2010
// Version		: Beta
// Target MCU	: Atmel AVR series
//
//
// This code is distributed under the GNU Public License
// which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************

/*  
	Disclaimer:
	This code is highly beta and might lack some funcionality and/or documentation.
	Furthermore the code has not been properly tested.
	Nevertheless, please expect most functions to work.
	

	I will not take any responsibility or what so ever for the functionality of the LSU7S1011A SED1530 driver.
	If you wreck anything - it's your own business...
	Modify this code and contribute freely.

  special thank to Marc Meise for his fundamental works to this project
*/


#ifndef SED1530_H

//A very simple delay-function, needed for initialisation of the SED1530
extern void delay (void);

//Init for the SPI, called by the Init_SED1530 routine
extern void Init_SPI (void);

//Init for the SED1530 - has to be called once
extern void Init_SED1530 (void);

//Clears the entire display of the E0855-2 icluding all Symbols
extern void SED_clear (void);

//Clears a Page (line)
//syntax: SED_clear_page (Page-Number (0to3));
extern void SED_clear_page (uint8_t Page);

//Set the column (invisible cursor) to the left-most visible position of the E0855-2
extern void SED_Home_Col (void);

//Prints a Charcter at the defined column and page-position
//Syntax: SED_putc ('character') or SED_putc (ASCII_Code)
extern void SED_putc (char character);

//Prints a string at the defined column and page-position
//Syntax: SED_puts ("String");
extern void SED_puts (char *string);

//Prints a string at the defined column and page-position out of Flash-Memory
//Syntax: SED_puts (Variable);
extern void SED_puts_p (char *s_progmem);

//Prints a numeric value or numeric variable at the defined column and page-position
//Syntax: SED_puts_num (Value); SED_puts_num (Variable);
extern void SED_puts_num (uint16_t numeric);

//Changes to the next Page and set the column (invisible cursor) to the left-most visible position of the E0855-2
//The lower Page is 0, and the top-most is 3
extern void SED_newline(void);

//Display one of ten graphic-images from the file PICS.c
//See PICS.c for information.
//Syntax: SED_showPIC (Number);
extern void SED_showPIC (uint8_t picture);

//Set Page and Column
//Sytax: SED_set_XY(Page [0-3],Column[0-95]);
extern void SED_set_XY(uint8_t Page, uint8_t Column);

//Send SED1530 to sleep
extern void SED_sleep (void);

//Wake-up SED1530 out of sleep-mode
extern void SED_wakeup (void);

//Set or change the contrast of the display ( steps: 0 to 32 -> standard=15=medium)
extern void SED_contrast (uint8_t contrast);

//Draw a bytepattern  (X-column, Y-Page, Bytepattern [i.e. 0b10111101])
//Lower left corner is 0,0
extern void SED_pixel(uint8_t X, uint8_t Y, uint8_t pixelbyte);


//print userdefinable character, defined in array "usrchar" at the curren position.
//Take a look at font.c
//character=position of character in array (0-7)
void SED_uchar(uint8_t character);						//added by P.L.



/*
  Scheme:
  BIT Column 0-95
P 7
A 6
G 5
E 4
3 3
  2
  1
  0
P 7					
A 6
G 5						  |-------Column
E 4						  |  |----Page
2 3						  |  |  |-Bytepattern "pixelbyte"
  2						  |  |  |
  1	   Example: SED_pixel(5, 1, 0b01100111);
  0   >5< here is column 5 in page 1
P 7	   0 \
A 6    1  |	
G 5    1  |
E 4    0  > the "pixelbyte" of the example
1 3    0  |
  2    1  |
  1    1  |
  0    1 /
P 7
A 6
G 5
E 4
0 3
  2
  1
  0
*/ 
#endif
